using DrWatson
@quickactivate "dg1d"
using dg1d

prms = Parameters(
  N = 6,
  K = 40,
  CFL = 0.5,
  tstart = 0.0,
  tend = 0.4,
  quadr = :LGL_lumped,
  scheme = :RK4,
  L = 1.0,
)

element = SpectralElement(prms)
mesh = Mesh(element, prms)
u0 = ScalarEq.initial_data(mesh, prms, :bump)
bc = ScalarEq.PeriodicBdryConditions()
# bc = ScalarEq.BdryConditions((t)->cos(4.0*t*2.0*π/prms.L)-1, which_bdry=(Eq.v() >= 0) ? LHS : RHS)
pblm = Problem(mesh=mesh, prms=prms, element=element, Eq=dg1d.AdvectionEq.Eq, bdryconds=bc)

# with AV
# av = nothing
av = AV.MDA(pblm, cmax=1.0)
# av = AV.MDH(pblm, cmax=cmax)
# av = AV.DB(pblm, cmax=cmax, cb=1.0)
# av = AV.EV(pblm, cmax=1.0, ce=1.0)
u = ScalarEq.evolve_problem(u0, pblm, av)

ScalarEq.plot(u, pblm, av)
