using DrWatson
@quickactivate "dg1d"
using dg1d

prms = Parameters(
  N = 7,
  K = 80,
  CFL = 0.2,
  tstart = 0.0,
  tend = 0.3,
  quadr = :LGL_lumped,
  scheme = :SSPRK3,
  L = 1.0,
)

element = SpectralElement(prms)
mesh = Mesh(element, prms)
u0 = ScalarEq.initial_data(mesh, prms, :sine)
bc = dg1d.PeriodicBdryConditions()
pblm = Problem(mesh=mesh, prms=prms, element=element, 
               Eq=dg1d.BurgersEq.Eq, bdryconds=bc)

# with AV
# av = nothing
av = AV.MDA(pblm, cmax=1.0)
# av = AV.MDH(pblm, cmax=cmax)
# av = AV.DB(pblm, cmax=cmax, cb=0.5)
# av = AV.EV(pblm, cmax=cmax, ce=0.15)
u = ScalarEq.evolve_problem(u0, pblm, av)

ScalarEq.plot(u, pblm, av)
