export BDRY, LHS, RHS
@doc """
Boundary identifiers.
"""
@enum BDRY LHS RHS


@doc """
Boundary condition interface, should implement the following methods

TODO Think again if this makes sense.
# - ()(t, args...) ... compute boundary value
"""
abstract type Abstract_BC end
# function (bc::Abstract_BC)(t, args...) end


#######################################################################
#                  Boundary Condition implementions                   #
#######################################################################


export Dirichlet_BC
@doc """
Dirichlet boundary condition, bdryval might be time dependent.
"""
struct Dirichlet_BC <: Abstract_BC 
  bdryval::Function
end
# make Dirichlet_BC callable
(bc::Dirichlet_BC)(args...) = bc.bdryval(args...)


@doc """
Constructor for Dirichlet BC with constant boundary value.
"""
Dirichlet_BC(bdry_val::Float64) = Dirichlet_BC((t)->bdry_val)


export Outflow_BC
@doc """
Outflow boundary condition.
"""
struct Outflow_BC <: Abstract_BC end


#######################################################################
#                    Boundary condition collection                    #
#######################################################################


abstract type Abstract_BdryConditions end


export BdryConditions
@doc """
Collection of boundary conditions on simulation domain.
"""
struct BdryConditions <: Abstract_BdryConditions
  lhs::Abstract_BC
  rhs::Abstract_BC
  # TODO Verify if lhs and rhs have same function signature
  BdryConditions(lhs::Abstract_BC, rhs::Abstract_BC) = new(lhs, rhs)
end


export PeriodicBdryConditions
@doc """
Periodic boundary conditions for simulation domain.
"""
struct PeriodicBdryConditions <: Abstract_BdryConditions end


@doc """
Get periodicity trait of bdry condition collection.
"""
isperiodic(bc::Abstract_BdryConditions) = false
isperiodic(bc::PeriodicBdryConditions) = true



# TODO This only makes sense for scalar equations.
@doc """
Constructor for standard bdry condition setup where one bdry is a
of Dirichlet type and the other of outflow type.
"""
function BdryConditions(bc::Function; which_bdry_is_dirichlet::BDRY)
  if which_bdry_is_dirichlet == LHS
    return dg1d.BdryConditions(lhs=dg1d.Dirichlet_BC(bc), rhs=dg1d.Outflow_BC())
  else
    return dg1d.BdryConditions(lhs=dg1d.Outflow_BC(), rhs=dg1d.Dirichlet_BC(bc))
  end
end
