using Gnuplot
include("gnuplot.jl")

const plotdir = normpath("$(@__DIR__)/../plots/")


function start_session(session) 
  @gp session :-
end


function end_session(session)
  @gp :- session
end


function reset_session(session)
  Gnuplot.reset(Gnuplot.getsession(session))
end


function set_terminal(session) 
  @gp :- session "set terminal qt size 960,720 font ',10' noraise" :-
end


function cmd_session(cmd, session)
  @gp :- session cmd :-
end


function plot(data; session=:default, plotopts="with lines linewidth 2")
    mydata = "\$MyData" * string(abs(rand(Int)))   # use a random id
    _, cols = size(data)
    @gp :- session mydata=>(data[:,1],data[:,2]) :-
    for idx in 2:cols
      plotstr = "plot $mydata using 1:$idx "
      plotstr *= plotopts
      @gp :- session plotstr :-
    end # for
end # function


# TODO Fix default value for keys, because ommitting it does not work.
function dg_plot(data::AbstractVector, mesh; session=:default,
    keys=Vector{String}(), plotopts="", plotcfg="")

  reset_session(session)
  start_session(session)
  set_terminal(session)

  if length(keys) != 0
    @assert length(keys) == length(data) "Expected equal number of data sets and key entries!"
    @gp :- session "set key" :-
  end
  @gp :- session plotcfg :-

  @unpack K, x, L = mesh
  N = length(data)
  pointidx = 2
  pointnumber = ceil(L/20) # two points per element
  for idx = 1:N
    opts = plotopts
    d = data[idx]
    K = size(d)[2]
    opts *= "with linespoints pointtype $pointidx " *
            "pointnumber $pointnumber pointsize 1 notitle"
    for k = 1:K
      plot(hcat(x[:,k], d[:,k]), session=session, plotopts=opts)
    end
    if length(keys) != 0
      # add ghost key entry with title
      key = keys[idx]
      plotstr = "plot NaN with linespoints pointtype $pointidx pointsize 3 " *
                "linecolor rgb 'black' linewidth 2 title '$key'"
      pointidx += 2
      @gp :- session plotstr :-
    end
  end

  end_session(session)
end


function save_png(filename, session=:default)
  Gnuplot.save(session, term="pngcairo", output = "$plotdir/$filename.png")
end
