@doc """
Diffusion Based Artificial Viscosity model, cf. Yu, Hesthaven, 2019.
"""
@with_kw mutable struct DB <: Abstract_AV @deftype Real
  cmax
  cb
  μ::AbstractArray
end


@doc """
Construct `DB` object based on `mesh`.
"""
function DB(prms::Parameters, mesh::Mesh)
  @unpack x = mesh
  μ0 = zeros(Float64, size(x))
  return DB(prms.cmax, prms.c1, μ0)
end


function compute(u, v, t, mesh::Mesh, eq::Abstract_Eq, av::DB)
  @unpack element, K, L, invjac = mesh
  @unpack D, N = element
  @unpack cmax, cb = av

  h = L / (K * N)

  maxabs_∇u = vec(maximum(abs.(D * u .* invjac), dims=1))
  μ_b = cb * (h / N)^2 * maxabs_∇u

  maxabs_v = vec(maximum(abs.(v), dims=1))
  μ_max = @. cmax * h / N * maxabs_v

  μ = min.(μ_b, μ_max)

  return μ
end
