abstract type Abstract_EulerEq <: Abstract_SystemOfEqs end
@with_kw struct Eq <: Abstract_EulerEq
  eos::EoS.IdealGasEoS
end


flux_ρ(ρ, q, E, p)  = q
flux_q(ρ, q, E, p)  = q^2 / ρ + p
flux_E(ρ, q, E, p)  = q / ρ * (E + p)


function flux(eq::Eq, ρ, q, E, x)
  p = EoS.pressure(eq.eos, ρ, q, E, x)
  f_ρ = flux_ρ.(ρ, q, E, p)
  f_q = flux_q.(ρ, q, E, p)
  f_E = flux_E.(ρ, q, E, p)
  return f_ρ, f_q, f_E
end


function flux(eq::Eq, u, x)
  ρ, q, E = unpack(u, x)
  return flux(eq, ρ, q, E, x)
end


function abs_v(eq::Eq, ρ, q, E, x)
  @unpack eos = eq
  p = EoS.pressure.(Ref(eos), ρ, q, E, x)
  return @. abs(q / p) + sqrt(abs(eos.γ * p / ρ))
end


abs_v(eq::Eq, u, x) = abs_v(eq, unpack(u, x)..., x)


function maxabs_v(eq::Eq, u, x)
  v = abs_v(eq, u, x)
  return maximum(v[:])
end


@doc """
Pack variables ρ, q, E into matrix u.

TODO Think about dispatch Parameters.@unpack macro.
"""
function pack(ρ::tA, q::tA, E::tA) where {tA<:AbstractArray}
  return vcat(ρ, q, E)
end


@doc """
Unpack variables ρ, q, E from matrix u.

TODO Think about dispatch Parameters.@unpack macro.
"""
function unpack(u::AbstractArray, x::AbstractArray)
  Npts, K = size(x)
  ρ = view(u, 1:Npts, 1:K)
  q = view(u, Npts+1:2*Npts, 1:K)
  E = view(u, 2*Npts+1:3*Npts, 1:K)
  return ρ, q, E
end
