# function evolve(prms::EulerEq_Parameters)
function evolve()

  N = 7
  K = 80
  quadr = :LGL
  CFL = 0.2
  xmin = -5.0
  xmax = 5.0
  # xmin = -1.0
  # xmax = 1.0
  # type_bdryconditions = :periodic
  # type_initialdata = :smooth_isentropic_flow
  type_bdryconditions = :from_initialdata
  type_initialdata = :shu_osher_shock_tube_v2
  tstart = 0.0
  tend = 0.1
  # γ = 3.0
  γ = 1.4
  av_method = :MDA
  av_cmax = 1.0
  av_c1 = nothing
  av_c2 = nothing
  scheme = :RK4


  element = SpectralElement(N, quadr)
  mesh = dg1d.Mesh(element, K, xmin, xmax, CFL)
  eq = Eq(EoS.IdealGasEoS(γ))

  u0 = initialdata(type_initialdata, mesh, eq, tstart)
  bcs = bdryconditions(type_bdryconditions, u0, mesh)

  av_prms = AV.Parameters(av_method, av_cmax, tstart, av_c1, av_c2)
  av = AV.make_AV(av_prms, mesh)

  ode!(du, u, t) = rhs!(du, u, t, mesh, eq, bcs, av)
  function dt(u::AbstractMatrix, t)
    @unpack x = mesh
    vmax = maxabs_v(eq, u, x)
    return AV.timestep(u, mesh, eq, av)
  end
   
  # interpolation matrix for smoothing operation
  intrp_order = 1
  intrp_element = dg1d.SpectralElement(intrp_order, quadr)
  intrp_from  = mesh.element.z
  intrp_to    = intrp_element.z
  intrp_mat   = dg1d.lagrange_interp_matrix(intrp_from, intrp_to)

  # post hook run after each full RK step
  function hook(u, t)
    ρ, q, E = unpack(u, mesh.x)
    p = EoS.pressure.(Ref(eq.eos), ρ, q, E, mesh.x)
    f_ρ = flux_ρ(ρ, q, E, p)
    absv = abs_v(eq, ρ, q, E, mesh.x)
    μ = AV.compute(ρ, absv, t, mesh, eq, av)
    av.μ .= dg1d.smooth_C0(μ, intrp_mat)
    return true
  end
  tspan = (tstart, tend)
  u_num = dg1d.evolve_ERK(ode!, u0, dt, tspan, scheme, post_hook=hook)

  plot(u_num, mesh, eq, av)

  return u_num
end
