function initialdata(type::Symbol, mesh::Mesh, eq::Eq, tstart::Float64 = 0)
  @unpack x, L, xmin, xmax = mesh
  @unpack eos = eq
  @unpack γ = eos

  tuple_data = if type === :smooth_isentropic_flow
    # Glaubitz, PhD thesis 2020, p. 219
    @assert xmin == -1.0 && xmax == 1.0
    @assert γ == 3.0
    ρ0 = @. 1.0 + 0.5 * sin(π * x)
    v0 = @. 0.0 * x
    p0 = @. ρ0^γ
    q0 = @. ρ0 * v0
    E0 = @. p0 / (γ - 1.0) + 0.5 * ρ0 * v0^2
    (ρ0, q0, E0)
  elseif type === :sod_shock_tube
    @assert xmin == -1.0 && xmax == 1.0
    @assert γ == 1.4
    ρ0 = [ (xi < 0.0) ? 1.0 : 0.125  for xi in x ]
    v0 = [ (xi < 0.0) ? 0.0 : 0.0    for xi in x ]
    p0 = [ (xi < 0.0) ? 1.0 : 0.1    for xi in x ]
    q0 = @. ρ0 * v0
    E0 = @. p0 / (γ - 1.0) + 0.5 * ρ0 * v0^2
    (ρ0, q0, E0)
  elseif type === :shu_osher_shock_tube_v1
    # taken from Glaubitz 2020, PhD thesis
    # results do not agree, I believe that ID is wrong
    @assert xmin == 0.0 && xmax == 4.0
    @assert γ == 1.4
    ρ0 = [ (xi <= 2.5) ? 3.857143 : 1.0 + 0.2 * sin(10*π*xi) for xi in x ]
    v0 = [ (xi <= 2.5) ? 2.629369 : 0.0    for xi in x ]
    p0 = [ (xi <= 2.5) ? 10.333333 : 1.0    for xi in x ]
    q0 = @. ρ0 * v0
    E0 = @. p0 / (γ - 1.0) + 0.5 * ρ0 * v0^2
    (ρ0, q0, E0)
  elseif type === :shu_osher_shock_tube_v2
    # taken from Yu, Hesthaven, 2019
    @assert xmin == -5.0 && xmax == 5.0
    @assert γ == 1.4
    ρ0 = [ (xi <= -4.0) ? 3.857143 : 1.0 + 0.2 * sin(5*xi) for xi in x ]
    v0 = [ (xi <= -4.0) ? 2.629369 : 0.0    for xi in x ]
    p0 = [ (xi <= -4.0) ? 10.333333 : 1.0    for xi in x ]
    q0 = @. ρ0 * v0
    E0 = @. p0 / (γ - 1.0) + 0.5 * ρ0 * v0^2
    (ρ0, q0, E0)
  else
    error("Initial data of type $type not implemented!")
  end

  u0 = pack(tuple_data...)
  return u0
end
