using dg1d
using Plots


function plot(u, mesh::Mesh, eq::Abstract_EulerEq, av::Union{AV.Abstract_AV,Nothing}=nothing)
  @unpack x, K = mesh

  Plots.gr()

  nplts = 3
  if !isnothing(av)
    nplts += 1
  end

  xlabel = "x"
  plts = [ Plots.plot() for _ in 1:nplts ]
  ρ, q, E = EulerEq.unpack(u, x)
  v = @. q / ρ
  p = EoS.pressure.(Ref(eq.eos), ρ, q, E, x)
  vars = [ ρ, v, p ]
  ylabels = [ "ρ(t,x)", "v(t,x)", "p(t,x)" ]

  if !isnothing(av)
    push!(vars, av.μ)
    push!(ylabels, "μ(t,x)")
  end

  for (plt, var, ylabel) in zip(plts, vars, ylabels)
    for k in 1:K
      Plots.plot!(plt, x[:,k], var[:,k], label=false)
    end
    Plots.plot!(plt, ylabel=ylabel)
  end

  plt = Plots.plot(plts..., layout=(nplts,1), size=(1200,1200))
  Plots.gui(plt)

  return
end


function plot_characteristics(u, mesh::Mesh, eq::Abstract_EulerEq, av::Union{AV.Abstract_AV,Nothing}=nothing)
  @unpack x, K = mesh

  xlabel = "x"
  vars = EulerEq.unpack(u, x)
  ρ, q, E = vars
  v = @. q / ρ
  p = EoS.pressure.(Ref(eq.eos), ρ, q, E, x)
  c2 = @. eq.eos.γ * p / ρ
  # dg1d.purge_zeros!(c2)
  c = @. sqrt(c2)

  λs = [ v - c, v, v + c ]

  Plots.gr()
  plt = Plots.plot(size=(1200,1200), xlabel="x", title="Characteristics")
  for λ in λs
    for k in 1:K
      Plots.plot!(x[:,k], λ[:,k], label=false)
    end
  end

  Plots.gui()

  return
end
