function rhs!(dU, U, t, mesh::Mesh, eq::Abstract_EulerEq, bcs::Abstract_BdryConditions, av::Abstract_AV)
  @unpack dl, K, x, element = mesh
  @unpack N = element
  @unpack μ = av

  # setup
  ρqE = unpack(U, x)
  dρqE = unpack(dU, x)
  f_ρqE = [ flux_ρ, flux_q, flux_E ]
  Q = zeros(size(U))
  q_ρqE = unpack(Q, x)

  isperiodic = dg1d.isperiodic(bcs)
  ρqE_lhs, ρqE_rhs = zeros(3), zeros(3)
  if !isperiodic
    bc_ρqE = [ bcs.ρ, bcs.q, bcs.E ]
    for (i, (u, bc)) in enumerate(zip(ρqE, bc_ρqE))
      ρqE_lhs[i], ρqE_rhs[i] = dg1d.bdry_flux(u, t, bc)
      ρqE[i][1], ρqE[i][end] = ρqE_lhs[i], ρqE_rhs[i]
    end
  end

  ## auxiliary equation: 0 = q - ∂x u
  for (du, u, f, q, u_lhs, u_rhs) in zip(dρqE, ρqE, f_ρqE, q_ρqE, ρqE_lhs, ρqE_rhs)
    intf_u = dg1d.to_interface(u, mesh)
    nu_lhs, nu_rhs = dg1d.central_flux(intf_u)
    if !isperiodic
      nu_lhs[1], nu_rhs[end] = u_lhs, u_rhs
    end
    dg1d.rhs_weak!(q, -u, -nu_lhs, -nu_rhs, mesh)
  end

  max_v = maxabs_v(eq, U, x)
  p = EoS.pressure.(Ref(eq.eos), ρqE..., x)

  # evaluate residual for each conserved variable
  ## reduced equation: 0 = ∂t u + ∂x F - ∂x μ q
  for (du, u, f, q) in zip(dρqE, ρqE, f_ρqE, q_ρqE)
    F = f.(ρqE..., p)
    G = -μ .* q
    intf_u  = dg1d.to_interface(u, mesh)
    intf_F  = dg1d.to_interface(F, mesh)
    intf_G  = dg1d.to_interface(G, mesh)
    nF_lhs, nF_rhs = dg1d.lax_friedrich_flux(intf_F, intf_u, max_v)
    nG_lhs, nG_rhs = dg1d.central_flux(intf_G)
    if !isperiodic
      nq_lhs, nq_rhs = dg1d.bdry_flux_LDG_variable(q, t, bcs)
      x_lhs, x_rhs = x[1], x[end]
      p_lhs, p_rhs = EoS.pressure(eq.eos, ρqE_lhs..., x_lhs), EoS.pressure(eq.eos, ρqE_rhs..., x_rhs)
      nF_lhs[1]   = f(ρqE_lhs..., p_lhs)
      nF_rhs[end] = f(ρqE_rhs..., p_rhs)
      nG_lhs[1]   = 0.0
      nG_rhs[end] = 0.0
    end

    nFG_lhs = nF_lhs + nG_lhs
    nFG_rhs = nF_rhs + nG_rhs
    FG = F + G
    dg1d.rhs_weak!(du, FG, nFG_lhs, nFG_rhs, mesh)
  end

  return
end
