function evolve()

  N = 5
  K = 40
  quadr = :LGL
  CFL = 0.2
  xmin = -1.0
  xmax = 1.0
  type_bdryconditions = :periodic
  type_initialdata = :smooth
  tstart = 0.0
  tend = 2.5
  γ = 3.0
  av_method = :MDH
  av_cmax = 1.0
  av_c1 = 1.0
  av_c2 = 1.0
  scheme = :RK4


  element = SpectralElement(N, quadr)
  mesh = dg1d.Mesh(element, K, xmin, xmax, CFL)
  eq = Eq(eos=EoS.SR_IdealGasEoS(γ))

  u0, p0 = initialdata(type_initialdata, mesh, eq, tstart)
  bcs = bdryconditions(type_bdryconditions, u0, mesh)

  av_prms = AV.Parameters(av_method, av_cmax, tstart, av_c1, av_c2)
  av = AV.make_AV(av_prms, mesh)

  ode!(du, u, t) = rhs!(du, u, t, p0, mesh, eq, bcs, av)
  function dt(u, t)
    @unpack x = mesh
    D, S, τ = unpack(u, x)
    ρ = Formulae.ρ.(D, S, τ, p0)
    v = Formulae.v.(D, S, τ, p0)
    ϵ = Formulae.ϵ.(D, S, τ, p0)
    vmax = abs_v.(Ref(eq), ρ, v, ϵ, p0, x)
    return AV.timestep(u, vmax, mesh, av)
  end
   
  # interpolation matrix for smoothing operation
  intrp_order = 1
  intrp_element = dg1d.SpectralElement(intrp_order, quadr)
  intrp_from  = mesh.element.z
  intrp_to    = intrp_element.z
  intrp_mat   = dg1d.lagrange_interp_matrix(intrp_from, intrp_to)

  # post hook run after each full RK step
  function hook(u, t)
    @unpack x = mesh
    D, S, τ = unpack(u, x)
    f_D = Formulae.flux_D.(D, S, τ, p0)
    ρ = Formulae.ρ.(D, S, τ, p0)
    v = Formulae.v.(D, S, τ, p0)
    ϵ = Formulae.ϵ.(D, S, τ, p0)
    vmax = abs_v.(Ref(eq), ρ, v, ϵ, p0, x)
    μ = AV.compute(ρ, vmax, t, mesh, eq, av)
    av.μ .= dg1d.smooth_C0(μ, intrp_mat)
    return true
  end
  tspan = (tstart, tend)
  u_num = dg1d.evolve_ERK(ode!, u0, dt, tspan, scheme, post_hook=hook)

  plot(u_num, p0, mesh, eq, av)

  return u_num
end
