function initialdata(type::Symbol, mesh::Mesh, eq::Eq, tstart::Float64 = 0)
  @unpack x, L, xmin, xmax = mesh
  @unpack eos = eq
  @unpack γ = eos

  ρ0, v0, p0 = if type === :smooth
    # Bugner, PhD thesis 2018
    @assert xmin == -1.0 && xmax == 1.0
    @assert γ == 3.0
    @assert typeof(eos) <: EoS.SR_IdealGasEoS
    vx = 0.2
    ρ0 = @. 1.0 + 0.2 * sin(2.0 * π * (x - vx * tstart))
    v0 = vx * ones(Float64, size(x))
    p0 = 1.0 * ones(Float64, size(x))
    ρ0, v0, p0
  elseif type === :shocktube
    # Bugner, PhD thesis 2018
    @assert xmin == 0.0 && xmax == 1.0
    @assert γ == 1.4
    @assert typeof(eos) <: EoS.SR_IdealGasEoS
    x0 = 0.5
    ρ0 = dg1d.stepdata(x, x0, 10.0, 1.0)
    v0 = dg1d.stepdata(x, x0, 0.0, 0.0)
    p0 = dg1d.stepdata(x, x0, 13.33, 1e-7)
    ρ0, v0, p0
  else
    error("Initial data of type $type not implemented!")
  end
  
  # TODO Implement interface to invert EoS
  ϵ0 = if typeof(eos) <: EoS.SR_IdealGasEoS
    @. p0 / ((γ - 1.0) * ρ0)
  end

  D0 = Formulae.D.(ρ0, v0, ϵ0, p0)
  S0 = Formulae.S.(ρ0, v0, ϵ0, p0)
  τ0 = Formulae.τ.(ρ0, v0, ϵ0, p0)
  u0 = pack(D0, S0, τ0)

  return u0, p0
end
