function rhs!(dU, U, t, p, mesh::Mesh, eq::Eq, bcs::Abstract_BdryConditions, av::Abstract_AV)
  @unpack dl, K, x, element = mesh
  @unpack N = element
  @unpack μ = av

  # setup
  DSτ = unpack(U, x)
  dDSτ = unpack(dU, x)
  f_DSτ = [ Formulae.flux_D, Formulae.flux_S, Formulae.flux_τ ]
  Q = zeros(size(U))
  q_DSτ = unpack(Q, x)

  isperiodic = dg1d.isperiodic(bcs)
  DSτ_lhs, DSτ_rhs = zeros(3), zeros(3)
  if !isperiodic
    bc_DSτ = [ bcs.D, bcs.S, bcs.τ ]
    for (i, (u, bc)) in enumerate(zip(DSτ, bc_DSτ))
      DSτ_lhs[i], DSτ_rhs[i] = dg1d.bdry_flux(u, t, bc)
      DSτ[i][1], DSτ[i][end] = DSτ_lhs[i], DSτ_rhs[i]
    end
  end

  ## auxiliary equation: 0 = q - ∂x u
  for (du, u, f, q, u_lhs, u_rhs) in zip(dDSτ, DSτ, f_DSτ, q_DSτ, DSτ_lhs, DSτ_rhs)
    intf_u = dg1d.to_interface(u, mesh)
    nu_lhs, nu_rhs = dg1d.central_flux(intf_u)
    if !isperiodic
      nu_lhs[1], nu_rhs[end] = u_lhs, u_rhs
    end
    dg1d.rhs_weak!(q, -u, -nu_lhs, -nu_rhs, mesh)
  end

  D, S, τ = DSτ
  p .= pressure.(Ref(eq), D, S, τ, p, x)
  ρ = Formulae.ρ.(D, S, τ, p)
  v = Formulae.v.(D, S, τ, p)
  ϵ = Formulae.ϵ.(D, S, τ, p)
  max_v = maxabs_v(eq, ρ, v, ϵ, p, x)
  
  # evaluate residual for each conserved variable
  ## reduced equation: 0 = ∂t u + ∂x F - ∂x μ q
  for (du, u, f, q) in zip(dDSτ, DSτ, f_DSτ, q_DSτ)
    F = f.(ρ, v, ϵ, p)
    G = - μ .* q
    intf_u  = dg1d.to_interface(u, mesh)
    intf_F  = dg1d.to_interface(F, mesh)
    intf_G  = dg1d.to_interface(G, mesh)
    nF_lhs, nF_rhs = dg1d.lax_friedrich_flux(intf_F, intf_u, max_v)
    nG_lhs, nG_rhs = dg1d.central_flux(intf_G)
    if !isperiodic
      x_lhs, x_rhs = x[1], x[end]
      p_lhs, p_rhs = p[1], p[end]
      D_lhs, S_lhs, τ_lhs = DSτ_lhs
      D_rhs, S_rhs, τ_rhs = DSτ_rhs
      p_lhs = pressure(eq, D_lhs, S_lhs, τ_lhs, p_lhs, x_lhs)
      p_rhs = pressure(eq, D_rhs, S_rhs, τ_rhs, p_rhs, x_rhs)
      ρ_lhs = Formulae.ρ(D_lhs, S_lhs, τ_lhs, p_lhs)
      ρ_rhs = Formulae.ρ(D_rhs, S_rhs, τ_rhs, p_rhs)
      v_lhs = Formulae.v(D_lhs, S_lhs, τ_lhs, p_lhs)
      v_rhs = Formulae.v(D_rhs, S_rhs, τ_rhs, p_rhs)
      ϵ_lhs = Formulae.ϵ(D_lhs, S_lhs, τ_lhs, p_lhs)
      ϵ_rhs = Formulae.ϵ(D_rhs, S_rhs, τ_rhs, p_rhs)
      nF_lhs[1]   = f(ρ_lhs, v_lhs, ϵ_lhs, p_lhs)
      nF_rhs[end] = f(ρ_rhs, v_rhs, ϵ_rhs, p_rhs)
      nG_lhs[1]   = 0.0
      nG_rhs[end] = 0.0
    end

    nFG_lhs = nF_lhs + nG_lhs
    nFG_rhs = nF_rhs + nG_rhs
    FG = F + G
    dg1d.rhs_weak!(du, FG, nFG_lhs, nFG_rhs, mesh)
  end

  return
end
