export BurgersEq
module BurgersEq


using dg1d
import dg1d: Abstract_ScalarEq, flux, maxabs_v
import Parameters: @with_kw, @unpack


@doc """
Burgers equation

  ∂t u + ∂x u^2 = 0

"""
struct Eq <: Abstract_ScalarEq end
export flux
flux(eq::Eq, u, x) = 0.5 * u.^2
maxabs_v(eq::Eq, u, x) = 0.5 * maximum(abs.(u)[:])


@with_kw mutable struct Parameters <: dg1d.AbstractParameters
  N::Int
  K::Int
  CFL::Real
  quadr::Symbol
  scheme::Symbol
  advection_v::Real
  av_method::Symbol
  av_cmax::Real
  av_c::Real
  problem::Symbol
  # initialdata::Symbol
  # tstart::Real
  # tend::Real
  # xmin::Real
  # xmax::Real
end


function evolve(prms::Parameters)
  @unpack_Parameters prms
  
  element = SpectralElement(N, quadr)
  mesh = Mesh(element, K, xmin, xmax, CFL)
  eq = Eq()
  u0 = dg1d.initial_data(id_type, mesh)
  bc = PeriodicBdryConditions()

  # with AV
  # av = missing
  av = dg1d.MDA(mesh, cmax=av_cmax)
  # av = dg1d.MDH(mesh, cmax=av_cmax)
  # av = dg1d.DB(mesh, cmax=av_cmax, cb=av_c)
  # av = dg1d.EV(mesh, tspan[1], cmax=av_cmax, ce=av_c)
  posthook = dg1d.PostHookAV(mesh, eq, av)

  ode!(du, u, t) = dg1d.rhs!(du, u, t, mesh, eq, bc, av)
  dt = dg1d.timestep(u0, mesh, eq)
  tspan = (tstart, tend)
  u_num = dg1d.evolve_ERK(ode!, u0, dt, tspan, scheme, post_hook = posthook.hook)

  # bc = ScalarEq.BdryConditions((t)->cos(4.0*t*2.0*π/prms.L)-1, which_bdry=(Eq.v() >= 0) ? LHS : RHS)
  # pblm = Problem(mesh=mesh, prms=prms, element=element, Eq=Eq, bdryconds=bc)

  return u_num
end


end # module
