module Analysis


using dg1d, JLD2, Plots

function gather_data()

  tstart = 0.0
  tend = 4.0
  N = collect(1:3)
  K = [10, 20, 40, 80]

  l2_err = zeros(length(N), length(K))

  for (in, n) in enumerate(N), (ik, k) in enumerate(K)

    try
      print("Running with N = $n, K = $k ⌛ ... ")
      u_num, u_ana, pblm = dg1d.AdvectionEq.evolve(n, k, tstart, tend)
      err = u_num - u_ana
      sqr_l2_err = dg1d.broken_inner_product(err, err, pblm.mesh)
      l2_err[in,ik] = sqrt(sqr_l2_err)
      # err[in,ik] = maximum(abs.(e[:]))
      println("✅")
    catch
      println("❌")
      err[in,ik] = NaN
    end

  end

  output_dir = dg1d.get_output_dir()
  filename = "cvg_advection_sine.jld"
  filename = joinpath(output_dir, filename)

  jldopen(filename, "w") do file
    write(file, "N", N)
    write(file, "K", K)
    write(file, "l2_err", l2_err)
  end

  return
end



function load_data()

  output_dir = dg1d.get_output_dir()
  filename = "cvg_advection_sine.jld"
  filename = joinpath(output_dir, filename)

  N = nothing
  K = nothing
  l2_err = nothing
  jldopen(filename, "r") do file
    N = read(file, "N")
    K = read(file, "K")
    l2_err = read(file, "l2_err")
  end

  return N, K, l2_err
end 



function plot_data()

  Plots.gr()
  theme(:ggplot2, markersize=1.5)

  N, K, l2_err = load_data()
  L = 2.0 * π

  plt = plot()
  for (i,n) in enumerate(N)
    plot!(plt, K, abs.(l2_err[i,:]); yaxis=(:log,), xaxis=(:log))

    y = @. log(l2_err[i,:])
    h = @. L / K
    x = @. log(h)
    _, _, Npoh = dg1d.least_sqr_linear(x, y)
    println("analytic N = $n, numerical N ≈ $(Npoh)")
  end

  display(plt)

end



end
