# abstract type Abstract_Problem end
# # this should work
# Abstract_Problem(pblm::Type{T}) where {T<:Abstract_Problem} = pblm()
# # not sure if this works
# Abstract_Problem(pblm) = Abstract_Problem(pblm)
# # what I really would like to have is to throw an error saying
# # that this problem is not implemented yet
#
# # list of implemented problems
# struct Sine                   <: Abstract_Problem end
# struct Bump                   <: Abstract_Problem end
# struct SmoothBump             <: Abstract_Problem end
# struct Gaussian               <: Abstract_Problem end
# struct PeriodicSine           <: Sine end
# struct PeriodicBump           <: Bump end
# struct PeriodicSmoothBump     <: SmoothBump end
# struct PeriodicGaussian       <: Gaussian end
# struct ReflectiveSine         <: Sine end
# struct ReflectiveBump         <: Bump end
# struct ReflectiveSmoothBump   <: SmoothBump end
# struct ReflectiveGaussian     <: Gaussian end


@doc """
Query initialdata by type for a given mesh. tstart can be used to offset
analytical initial data.
"""
function initialdata(type::Symbol, mesh::Mesh; tstart::Real = 0)
  @unpack x, L, xmin = mesh

  if type === :sine
    u0 = @. sin(2.0 * π * (x + 2.0 * π * tstart) / L)
  elseif type === :bump
    xl = xmin + L * 0.25
    xr = xmin + L * 0.75
    u0 = [ (xi < xl || xi > xr) ? 0.0 : 1.0 for xi in x ]
  elseif type === :smooth_bump
    xl = xmin + L * 0.10
    xr = xmin + L * 0.90
    gap_x = 0.3 * L
    u0 = zeros(size(x))
    for (i, xi) in enumerate(x)
      if xi < xl || xi > xr
        u0[i] = 0.0
      elseif xi > xl+gap_x && xi < xr-gap_x
        u0[i] = 1.0
      elseif xi >= xl && xi <= xl+gap_x
        u0[i] = sin( (xi - xl) / gap_x * 0.5 * π)^2
      elseif xi >= xr-gap_x && xi <= xr
        u0[i] = 1.0 - sin( (xi - (xr - gap_x)) / gap_x * 0.5 * π)^2
      end
    end
  elseif type === :gaussian
    x0 = xmin + 0.5 * L
    u0 = @. exp(- (x - x0)^2 * 50.0)
  else
    error("No initial data of type '$type' implemented!")
  end

  return u0
end
