# struct with keyword constructor, default values and assertions enabled
# (all added by @with_kw)
@with_kw struct Parameters_Base <: AbstractParameters
  number = 1
  int_number::Int = 2
  string::String = "blabla"

  @assert number > 0
  @assert int_number >= 2
end


# struct with invalid fields
@with_kw struct Parameters_Invalid <: AbstractParameters
  symbol::Symbol = :invalid
end


# struct with nested substructs
# contains check on parameters of nested struct
@with_kw struct Parameters_Nested <: AbstractParameters
  nested_pars::Parameters_Base

  @assert nested_pars.number <= 2 # make sure number is not too big
end


# allow any subtype of AbstractParameters for the field par
@with_kw struct Parameters_Dummy <: AbstractParameters
  pars::AbstractParameters
end


function get_output_dir()
  thisdir = dirname(@__FILE__)
  output_dir = "$thisdir/../outputs/"
  output_dir = normpath(output_dir)
  if !isdir(output_dir)
    mkdir(output_dir)
  end
  return output_dir
end


@testset "Parameters" begin


@testset "Keyword construction and IO of parameter structs" begin

  # valid calls
  pars_base = Parameters_Base()
  pars_base = Parameters_Base(number = 1.23, string = "some string")
  pars_nested = Parameters_Nested(nested_pars=pars_base)
  Parameters_Dummy(pars=pars_base)

  # note that this works, but symbols should not be used, because they
  # are not specified in TOML
  Parameters_Invalid()

  # invalid values
  @test_throws Exception Parameters_Base(number = -1)
  @test_throws Exception Parameters_Base(string = 1)
  # trigger assertion on member of member
  pars_base = Parameters_Base(number = 3)
  @test_throws Exception Parameters_Nested(nested_pars = pars_base)
  # invalid type
  @test_throws Exception Parameters_Dummy(pars=10)

  thisdir = dirname(@__FILE__)
  output_dir = get_output_dir()

  # save and load
  save(pars_base, joinpath(output_dir, "base.toml"))
  base = load(Parameters_Base, joinpath(output_dir, "base.toml"))
  save(pars_nested, joinpath(output_dir, "nested.toml"))
  nested = load(Parameters_Nested, joinpath(output_dir, "nested.toml"))

end


end
